@@ -39,6 +39,7 @@
 #include <linux/wireless.h>
 #include <net/iw_handler.h>	// new driver API
 
+#include "compat.h"
 #include <net/ieee80211.h>
 
 #ifdef CONFIG_IPW2100_MONITOR
@@ -46,6 +47,9 @@
 #endif
 
 #include <linux/workqueue.h>
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,15)
+#include <linux/mutex.h>
+#endif
 
 struct ipw2100_priv;
 struct ipw2100_tx_packet;
@@ -584,11 +588,19 @@ struct ipw2100_priv {
 	struct tasklet_struct irq_tasklet;
 
 	struct workqueue_struct *workqueue;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
 	struct work_struct reset_work;
 	struct work_struct security_work;
 	struct work_struct wx_event_work;
 	struct work_struct hang_check;
 	struct work_struct rf_kill;
+#else
+	struct delayed_work reset_work;
+	struct delayed_work security_work;
+	struct delayed_work wx_event_work;
+	struct delayed_work hang_check;
+	struct delayed_work rf_kill;
+#endif
 
 	u32 interrupts;
 	int tx_interrupts;
@@ -596,8 +608,8 @@ struct ipw2100_priv {
 	int inta_other;
 
 	spinlock_t low_lock;
-	struct semaphore action_sem;
-	struct semaphore adapter_sem;
+	struct mutex action_mutex;
+	struct mutex adapter_mutex;
 
 	wait_queue_head_t wait_command_queue;
 
